<?php
/**
 * OSA_Session.php
 *
 * 	Overrides the system Session class to support object
 *    storage. If CodeIgniter starts to support object storage we can simply
 *    remove the 2 funtions.
 *
 *    Moved all the Tillersoft specific code that was in Session.php to this file.
 *    Some of this code might not work in future CI upgrades. Please check this file for
 *    every CI upgrade.
 *
 *    if ( isset($_SERVER['HTTP_X_CMS_REQUEST_TYPE']) && strtolower($_SERVER['HTTP_X_CMS_REQUEST_TYPE']) == 'sync' )
 *
 * @author $Author: dtong $
 * @version $Id: Osa_Session.php,v 1.7 2012/05/28 05:24:56 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class Osa_Session extends CI_Session {
	private $cached_objects = array();

	function __construct($params=array()) {
		$CI = &get_instance();
		// We need to load the CI database library for admin_loadconfig.
		// This happens where CI first load the session.
		$CI->load->database();
		$CI->config->set_item('cms_lang_ignore_all', TRUE);
		$CI->load->library('admin/admin_loadconfig');
		$CI->config->set_item('cms_lang_ignore_all', FALSE);
		parent::__construct($params);
   }

   // An object needs to be serialize again. CI serializes the whole session.
   function set_userobject($newdata, $newval = '') {
	   $newval = serialize($newval);
   	return $this->set_userdata($newdata, $newval);
   }

   // Need to unserialize an user object. This also caches the objects.
   function userobject($item) {
   	if ( isset($this->cached_objects[$item]) ) {
   		return $this->cached_objects[$item];
   	}

   	if ( !isset($this->userdata[$item]) || empty($this->userdata[$item]) ) {
   		return FALSE;
   	}

		$this->cached_objects[$item] = unserialize($this->userdata[$item]);
		return $this->cached_objects[$item];
   }

   // We override the session write so we can clean the cache every time the seesion gets updated.
   // The update might not be related to the cached object but this is the safest.
   function sess_write() {
   	// Clean the cache regardless what is being updated.
   	$this->cached_objects = array();
   	parent::sess_write();
   }

   /**
   * Create a new session
   *
   * We copied this from CI and modified some code. Needs to be checked on every CI upgrade.
   *
   * @access	public
   * @return	void
   */
   function sess_create()
   {
   	$sessid = '';
   	while (strlen($sessid) < 32)
   	{
   		$sessid .= mt_rand(0, mt_getrandmax());
   	}

   	// To make the session ID even more secure we'll combine it with the user's IP
   	$sessid .= $this->CI->input->ip_address();

   	$this->userdata = array(
   							'session_id'	=> md5(uniqid($sessid, TRUE)),
   							'ip_address'	=> $this->CI->input->ip_address(),
   							'user_agent'	=> substr($this->CI->input->user_agent(), 0, 120),
   							'last_activity'	=> $this->now,
   							'user_data'		=> ''
   	);

   	/* Tillersoft Modification Begin */
   	// In codeigniter 2.0.3 changed session to use a longer match against the user_agent string
   	// but login page using database session that new user agent string exceed a 'user_agent' column
   	// and we can't update 'ci_sessions' table before logging into system
   	// so check system version, if the ci version lower than 2.0.3 use truncated user agent string
   	$cms_db_version = $this->CI->config->item('cms_db_version');
   	// old version of todcm has no system version config so use a truncated user agent
   	// or compare version by using strnatcasecmp - returns < 0 if first string is less than second string
   	if ( empty($cms_db_version) || strnatcasecmp($cms_db_version, '1.2.0') < 0 ){
   		$this->userdata['user_agent'] = substr($this->CI->input->user_agent(), 0, 50);
   	}
   	/* Tillersoft Modification End */

   	// Save the data to the DB if needed
   	if ($this->sess_use_database === TRUE)
   	{
   		$this->CI->db->query($this->CI->db->insert_string($this->sess_table_name, $this->userdata));
   	}

   	// Write the cookie
   	$this->_set_cookie();
   }

   /**
   * Update an existing session
   *
   * We can't use a new session id with async ajax calls so add $use_new_id flag to
   * tell to use new session id or not. The default is not to use a new id and retain the
   * old id. Since CI always call this function without a parameter so it will be always
   * old id unless TODCM calls sess_update with $use_new_id being true.
   *
   * @access	public
   * @return	void
   */
   /* Tillersoft Modification Begin */
   // Please update this funciton for all CI upgrades. All the code related to $use_new_id
   // is from Tilelrsoft.
   // function sess_update()
   function sess_update($use_new_id=FALSE)
   /* Tillersoft Modification End */
   {
   	// We only update the session every five minutes by default
   	if (($this->userdata['last_activity'] + $this->sess_time_to_update) >= $this->now)
   	{
   		return;
   	}

   	// Tillersoft - Skip the update if http header has the skip tag.
   	if ( $this->_skip_upate() ) {
   		return;
   	}

   	// Save the old session id so we know which record to
   	// update in the database if we need it
   	$old_sessid = $this->userdata['session_id'];
   	// Tillersoft - We first assign the new session id as the old id.
   	$new_sessid = $old_sessid;
   	// Tillersoft - We added this if statement.
   	if ( $use_new_id === TRUE ) {
	   	$new_sessid = '';
	   	while (strlen($new_sessid) < 32)
	   	{
	   		$new_sessid .= mt_rand(0, mt_getrandmax());
	   	}

	   	// To make the session ID even more secure we'll combine it with the user's IP
	   	$new_sessid .= $this->CI->input->ip_address();

	   	// Turn it into a hash
	   	$new_sessid = md5(uniqid($new_sessid, TRUE));
   	}

   	// Update the session data in the session data array
   	$this->userdata['session_id'] = $new_sessid;
   	$this->userdata['last_activity'] = $this->now;

   	// _set_cookie() will handle this for us if we aren't using database sessions
   	// by pushing all userdata to the cookie.
   	$cookie_data = NULL;

   	// Update the session ID and last_activity field in the DB if needed
   	if ($this->sess_use_database === TRUE)
   	{
   		// set cookie explicitly to only have our session data
   		$cookie_data = array();
   		foreach (array('session_id','ip_address','user_agent','last_activity') as $val)
   		{
   			$cookie_data[$val] = $this->userdata[$val];
   		}

   		$this->CI->db->query($this->CI->db->update_string($this->sess_table_name, array('last_activity' => $this->now, 'session_id' => $new_sessid), array('session_id' => $old_sessid)));
   	}

   	// Write the cookie
   	$this->_set_cookie($cookie_data);
   }

   private function _skip_upate() {
   	return ( isset($_SERVER['HTTP_X_CMS_SKIP_SESSION_UPDATE']) );
   }

}