<?php
/**
 * unit_buttons.php - render the buttons data
 *
 * @author $Author: dtong $
 * @version $Id: unit_buttons.php,v 1.35 2012/04/02 05:08:47 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/
if ( ! defined('BASEPATH')) exit('No direct script access allowed');
require_once ('unit_base.php');

class unit_buttons extends unit_base {
	static private $replace_patterns =
		array(parent::UNITID_PATTERN, parent::STAGEDIV_PATTERN, parent::STAGENUM_PATTERN, parent::COURSEID_PATTERN, parent::RESID_PATTERN);
	static private $replace_patterns_nores =
		array(parent::UNITID_PATTERN, parent::STAGEDIV_PATTERN, parent::STAGENUM_PATTERN, parent::COURSEID_PATTERN);

	private $imagepath = FALSE;

	function __construct() {
		//parent unit_base uses this object so don't call parent constructor
		//parent::__construct();
		$this->CI = & get_instance();
	}

	//Returns the button(s) html
	function render(&$buttons, $params) {
		static $is_pdf_mode = NULL;
		static $imagepath = FALSE;

		if ( empty($buttons) ) {
			//This is necessary since btnsort can return FALSE for readonly operation.
			return '';
		}
		$this->reset($buttons, $params);
		if ( !isset($this->CI->event_handle_main) ) {
			$this->CI->load->library('unit/event/event_handle_main');
		}
		if ( $is_pdf_mode === NULL ) {
			$is_pdf_mode = osa_pdf_mode();
		}
		//See if we need to skip the buttons for pdf and unit compare
		if ( $is_pdf_mode || $params->use_cssid) {
			//template tells which button to skip for pdf
			//if ( $this->getval('_skip4pdf',$button) === TRUE )
			//Just skip all buttons for PDF and stop using _skip4pdf
			return '';
		}

		if ( $this->imagepath === FALSE ) {
			$this->imagepath = osa_imagepath();
		}

		if ( isset($this->params->resid) && !empty($this->params->resid) ) {
			$replace_patterns = &self::$replace_patterns;
			$replace_values = array($this->params->unitid, $this->params->divid, $this->params->stagenum, $this->params->courseid, $this->params->resid);
		}
		else {
			$replace_patterns = &self::$replace_patterns_nores;
			$replace_values = array($this->params->unitid, $this->params->divid, $this->params->stagenum, $this->params->courseid);
		}

		// foreach ($buttons as $button) {
		foreach (array_keys($buttons) as $butkey) {
			$button = &$buttons[$butkey];
			//Check read,write access for each template object
			//The buttons are different, they have to be checked inside unit_buttons.php
			//If there is no access for a object then it will continue/skip to the next one even before the switch below
			if ( !$this->CI->event_handle_main->run($button, $params) ) {
				$this->html .= $this->CI->event_handle_main->html_error_msg;
				continue;
			}
			//Assign error msg from event_handle_main if any. This is empty string if no error occured.
			if ( !empty($this->CI->event_handle_main->html_error_msg) ) {
				$this->html .= $this->CI->event_handle_main->html_error_msg;
			}

			$action = isset($button['_action']) ? $button['_action'] : FALSE;
			$type = isset($button['_type']) ? $button['_type'] : FALSE;
			$empty_button = isset($button['_empty_button']) ? $button['_empty_button'] : FALSE;
			$link = isset($button['_link']) ? $button['_link'] : FALSE;
			$image = isset($button['_image']) ? $button['_image'] : FALSE;
			$alt_title = isset($button['_alt_title']) ? $button['_alt_title'] : '';
			$image_id = isset($button['_image_id']) ? $button['_image_id'] : FALSE;
			$jscript = isset($button['_jscript']) ? $button['_jscript'] : FALSE;
			$jevent = isset($button['_jevent']) ? $button['_jevent'] : FALSE;

			if ( !osa_valid_action($action, $this->params->action)) {
					if ( !(osa_valid_action($action, CMS_ACTION_MY) && $this->is_teacher()) )
					continue;
			}

			if ( $type === FALSE ) {
				$this->error(__METHOD__ . ' - Missing Type', $button);
			}

			if ( $empty_button === TRUE ) {
				$this->print_empty_button();
				continue;
			}

			if ( $link  === FALSE) {
				$this->error(__METHOD__ . ' - Missing Type', $button);
			}
			/* Don't need title, some old templates are still using this...
			$title = $this->getval('_title',$button);
			if ( !$title )
				$title = '';
			else
				$title = ' ' . $title;
			*/
			if ( !empty($image) ) {
				if ( !empty($alt_title) ) {
					$alt_title = "title=\"$alt_title\"";
				}

				if ( $image_id !== FALSE ) {
					$image_id = "id=\"$image_id\"";
				}
				if ( osa_url_full($image) ) {
					$image_url = &$image;
				}
				else {
					$image_url = "{$this->imagepath}{$image}";
				}
				$image = "<img {$image_id} {$alt_title} src=\"{$image_url}\" border=\"0\" />";
			}
			else {
				$image='';
			}

			if ( empty($image) ) {
				$this->error(__METHOD__ . ' - Missing image and title', $button);
				$this->html .= osa_buildererror();
				continue;
			}

			//Replace unitid and resid in the links and jscript from the template
			/* $link = str_replace(parent::UNITID_PATTERN, $this->params->unitid, $link);
			if ( property_exists($this->params, 'resid' ) && !empty($this->params->resid) ) {
				$link = str_replace(parent::RESID_PATTERN, $this->params->resid, $link);
			}
			$link = str_replace(parent::STAGEDIV_PATTERN, $this->params->divid, $link);
			$link = str_replace(parent::STAGENUM_PATTERN, $this->params->stagenum, $link);
			$link = str_replace(parent::COURSEID_PATTERN, $this->params->courseid, $link); */
			$link = str_replace($replace_patterns, $replace_values, $link);
			if ( $jscript !== FALSE ) {
				/* $jscript = str_replace(parent::UNITID_PATTERN, $this->params->unitid, $jscript);
				if ( property_exists($this->params, 'resid' ) && !empty($this->params->resid) ) {
					$jscript = str_replace(parent::RESID_PATTERN, $this->params->resid, $jscript);
				}
				$jscript = str_replace(parent::STAGEDIV_PATTERN, $this->params->divid, $jscript);
				$jscript = str_replace(parent::STAGENUM_PATTERN, $this->params->stagenum, $jscript);
				$jscript = str_replace(parent::COURSEID_PATTERN, $this->params->courseid, $jscript); */
				$jscript = str_replace($replace_patterns, $replace_values, $jscript);
				if ( !empty($jevent) ) {
					$jscript = " {$jevent}=\"{$jscript}\" ";
				}
				else {
					$jscript = " onClick=\"{$jscript}\" ";
				}
			}

			if ( empty($link) ) {
				$href = '';
			}
			else {
				$href = "href=\"{$link}\"";
			}

 			$this->html .= "<div class=\"unitbutton\">";
 			if ( empty($jscript) ) {
 				$this->html .= "<a $href class=\"buttonalink\" >{$image}</a>";
 			}
 			else {
 				$this->html .= "<a $href class=\"buttonalink\" {$jscript}>{$image}</a>";
 			}
 			$this->html .= '</div>';
		}
		if ( !$this->html ) {
			return '';
		}
		return "<nobr>{$this->html}</nobr>";
	}

	private function print_empty_button() {
		$this->html .= '<div class="unitbutton"><img border="0" src="' . $this->imagepath . 'none.gif" /></div>';
	}
}
?>