<?php
/**
 * common_db_model.php - Generic common/shared DB functions. Do not put non-generic DB code here.
 *
 * @author $Author: dtong $
 * @version $Id: common_db_model.php,v 1.8 2012/02/20 09:59:12 dtong Exp $
 * @copyright Copyright (c) 2010, Tiller Software Co., Ltd.
*/

class Common_db_model extends CI_Model {

	function __construct() {
		parent::__construct();
	}

	function select($table, $where_array=FALSE) {
		if ( !$table ) {
			return FALSE;
		}
		if ( !$where_array ) {
		$query = $this->db->get($table);
		}
		else {
			$query = $this->db->get_where($table, $where_array);
		}
		if ( $query->num_rows() <= 0 ) {
			return FALSE;
		}
		return $query->result();
	}

	function select_byid($table, $id) {
		if ( !osa_is_int_one($id) || !$table ) {
			return FALSE;
		}
		$res = $this->db->get_where($table, array('id' => $id));
		if ( $res->num_rows() != 1 ) {
			return FALSE;
		}
		$ret = $res->result();
		return $ret[0];
	}

	// $data can be object or array.
	// $required_fields has to be an array if not false.
	function insert($table, $data, $required_fields=FALSE) {
		if ( !$table || !$this->_valid_data_set($data, TRUE) ) {
			return FALSE;
		}
		if ( is_array($required_fields) && !empty($required_fields) ) {
			foreach ($required_fields as $field) {
				if (!array_key_exists($field, $data)) {
					osa_log(__METHOD__, 'Failed on required field check.', $table, $data, $required_fields);
					return FALSE;
				}
			}
		}
		$ret = $this->db->insert($table, $data);
		if ( $ret ) {
			return osa_db_lastinsertid($this->db);
		}
		return FALSE;
	}

	// $data can be object or array.
	// $valid_fields has to be an array if not false.
	function update($table, $data, $id, $valid_fields=FALSE) {
		if ( !$table || !osa_is_int_one($id) || !$this->_valid_data_set($data, TRUE) ) {
			return FALSE;
		}
		if ( is_array($valid_fields) && !empty($valid_fields) ) {
			foreach ($data as $key => $value) {
				if ( !in_array($key, $valid_fields)) {
					osa_log(__METHOD__, 'Failed on valid field check.', $table, $data, $valid_fields);
					return FALSE;
				}
			}
		}
		$this->db->where('id', $id);
		return $this->db->update($table, $data);
	}

	// Combines insert and update
	function modify($table, $data, $id=FALSE, $check_fields=FALSE) {
		if ( !osa_is_int_one($id) ) {
			return $this->insert($table, $data, $check_fields);
		}
		return $this->update($table, $data, $id, $check_fields);
	}

	function query($sql) {
		if ( empty($sql) ) {
			return FALSE;
		}
		$query = $this->db->query($sql);
		if ( $query->num_rows() <= 0 ) {
			return FALSE;
		}
		return $query->result();
	}

	// $id can be an integer or an array/object to specific where conditions.
	public function delete($table, $id) {
		if ( !$table ) {
			return FALSE;
		}
		if ( osa_is_int_one($id) ) {
			$id = array('id'=>$id);
		}
		if ( !$this->_valid_data_set($id) ) {
			return FALSE;
		}
		return $this->db->delete($table, $id);
	}

	// Checks to make sure the input data object or array contains data.
	// Also converts the object to an associate array if required. This is necessary
	// if the code needs to access the data elements so only needs to write the code
	// to handle array.
	public function _valid_data_set(&$data, $convert_object=FALSE) {
		if (!empty($data) && (is_object($data) || is_array($data))) {
			if ( $convert_object && is_object($data) ) {
				if ( ($data = get_object_vars($data)) ) {
					return TRUE;
				}
			}
			if ( is_array($data) && count($data) <= 0 ) {
				return FALSE;
			}
			if ( is_object($data) && !get_object_vars($data) ) {
				return FALSE;
			}
			return TRUE;
		}
		return FALSE;
	}
}